/*
 * $Id: ServerSelection.java 286 2005-12-03 01:07:16Z davidb $
 * 
 * Copyright (c) 2005 VeriSign. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. 3. The name of the author may not
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package se.rfc.unbound;

import java.util.List;

/**
 * This interface describes a plug-in system for handling "server selection"
 * for the Iterative- or Stub- Resolver modules. "Server selection" refers to
 * the process of choosing which nameserver target to contact first (or
 * second, third, etc) from a list of known, presumably equivalent targets.
 * 
 * @author davidb
 * @version $Revision: 286 $
 */
public interface ServerSelection
{
  /**
   * Given a query/response cycle against a given Host (nameserver), record
   * the latest round-trip-time.
   * 
   * @param host The nameserver that was queried.
   * @param rtt The round trip time, in milliseconds. A timeout is represented
   *          as -1.
   */
  public void setRTT(Host host, long rtt);
  
  /**
   * Given a particular Host, determine a wakeup timer value. The wakeup timer
   * value is, essentially, how long should the resolver wait before trying
   * the next target (while not giving up on the current target)?
   * 
   * @param host The nameserver in question.
   * @return The wakeup timer value, in milliseconds. A negative value means
   *         to not wakeup (that is, only timeout).
   */
  public int getWakeupTimer(Host host);
  
  /**
   * Given a list of potential nameserver targets for a particular query, put
   * them in the order that they should be queried in. Note that this list may
   * contain known invalid targets (e.g., lame servers, forbidden servers)
   * which will be skipped.
   * 
   * @param targets A list of Host objects representing potential targets for
   *          a given query stage.
   */
  public void orderTargets(List targets);
  
}
